"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withTouchableHandler = exports.getSkiaPath = void 0;
var _react = require("react");
var _context = require("../canvas/context");
var _getCirclePath = require("../utils/get-circle-path");
var _getRectPath = require("../utils/get-rect-path");
var _unwrapAnimatedValue = require("../utils/unwrap-animated-value");
const getSkiaPath = (key, props) => {
  'worklet';

  const unwrappedProps = (0, _unwrapAnimatedValue.unwrapAnimatedValueObject)(props);
  switch (key) {
    case 'Circle':
      return (0, _getCirclePath.getCirclePath)(unwrappedProps);
    case 'Rect':
      return (0, _getRectPath.getRectPath)(unwrappedProps);
    case 'RoundedRect':
      return (0, _getRectPath.getRoundedRectPath)(unwrappedProps);
    case 'Path':
      return unwrappedProps.path;
    default:
      return null;
  }
};
exports.getSkiaPath = getSkiaPath;
const withTouchableHandler = (Component, componentName) => {
  return _ref => {
    let {
      onStart: onStartProp,
      onActive: onActiveProp,
      onEnd: onEndProp,
      touchablePath,
      ...props
    } = _ref;
    const id = (0, _react.useId)();
    const ref = (0, _context.useTouchHandlerContext)();
    const onStart = (0, _react.useCallback)(event => {
      'worklet';

      return onStartProp === null || onStartProp === void 0 ? void 0 : onStartProp(event);
    }, [onStartProp]);
    const onActive = (0, _react.useCallback)(event => {
      'worklet';

      return onActiveProp === null || onActiveProp === void 0 ? void 0 : onActiveProp(event);
    }, [onActiveProp]);
    const onEnd = (0, _react.useCallback)(event => {
      'worklet';

      return onEndProp === null || onEndProp === void 0 ? void 0 : onEndProp(event);
    }, [onEndProp]);
    const isPointInPath = (0, _react.useCallback)(point => {
      'worklet';

      if (touchablePath) {
        return (0, _unwrapAnimatedValue.unwrapAnimatedValue)(touchablePath).contains(point.x, point.y);
      }
      if (!componentName) return false;
      const path = getSkiaPath(componentName, props);
      if (!path) {
        throw Error('No touchablePath provided');
      }
      return path.contains(point.x, point.y);
    }, [props, touchablePath]);
    (0, _react.useEffect)(() => {
      ref.value = {
        [`id:${id}`]: {
          isPointInPath,
          onStart,
          onActive,
          onEnd
        },
        ...ref.value
      };
      // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [id, isPointInPath, onActive, onEnd, onStart]);
    (0, _react.useEffect)(() => {
      return () => {
        var _ref$value;
        (_ref$value = ref.value) === null || _ref$value === void 0 ? true : delete _ref$value[`id:${id}`];
      };
    }, [id, props, ref, touchablePath]);
    return Component(props);
  };
};
exports.withTouchableHandler = withTouchableHandler;
//# sourceMappingURL=with-touchable-handler.js.map