import { Canvas as SkiaCanvas } from '@shopify/react-native-skia';
import React, { useEffect, useMemo, useRef, useState } from 'react';
import { Gesture, GestureDetector } from 'react-native-gesture-handler';
import { useSharedValue } from 'react-native-reanimated';
import { TouchHandlerContext } from './context';
const Canvas = _ref => {
  let {
    children,
    panGesture = Gesture.Pan(),
    timeoutBeforeCollectingRefs = 100,
    ...props
  } = _ref;
  // Instead of value, provide a subscribe method and reload the refs
  const touchableRefs = useMemo(() => {
    return {
      value: {}
    };
  }, []);
  const activeKey = useSharedValue([]);

  // This must be improved, it's a hack to wait for the refs to be loaded
  const [loadedRefs, prepareLoadedRefs] = useState({});
  const ref = useRef();
  useEffect(() => {
    ref.current = setTimeout(() => {
      prepareLoadedRefs(touchableRefs.value);
    }, timeoutBeforeCollectingRefs);
    return () => {
      clearTimeout(ref.current);
    };
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [timeoutBeforeCollectingRefs]);
  const mainGesture = panGesture.onBegin(event => {
    'worklet';

    const keys = Object.keys(loadedRefs);
    for (let i = 0; i < keys.length; i++) {
      const key = keys[i];
      const touchableItem = loadedRefs[key];
      const isPointInPath = touchableItem === null || touchableItem === void 0 ? void 0 : touchableItem.isPointInPath(event);
      if (isPointInPath && touchableItem !== null && touchableItem !== void 0 && touchableItem.onStart) {
        var _touchableItem$onStar;
        activeKey.value = [`${key}__${event.handlerTag}`];
        (_touchableItem$onStar = touchableItem.onStart) === null || _touchableItem$onStar === void 0 ? void 0 : _touchableItem$onStar.call(touchableItem, event);
      }
    }
  }).onUpdate(event => {
    'worklet';

    var _activatedKey$split, _touchableItem$onActi;
    const activatedKey = activeKey.value.find(key => key.includes(event.handlerTag.toString()));
    if (!activatedKey) {
      return;
    }
    const indexedKey = (_activatedKey$split = activatedKey.split('__')) === null || _activatedKey$split === void 0 ? void 0 : _activatedKey$split[0];
    if (!indexedKey) {
      return;
    }
    const touchableItem = loadedRefs[indexedKey];
    return touchableItem === null || touchableItem === void 0 ? void 0 : (_touchableItem$onActi = touchableItem.onActive) === null || _touchableItem$onActi === void 0 ? void 0 : _touchableItem$onActi.call(touchableItem, event);
  }).onFinalize(event => {
    'worklet';

    var _activatedKey$split2, _touchableItem$onEnd;
    const activatedKey = activeKey.value.find(key => key.includes(event.handlerTag.toString()));
    if (!activatedKey) {
      return;
    }
    const indexedKey = (_activatedKey$split2 = activatedKey.split('__')) === null || _activatedKey$split2 === void 0 ? void 0 : _activatedKey$split2[0];
    if (!indexedKey) {
      return;
    }
    const touchableItem = loadedRefs[indexedKey];
    activeKey.value = activeKey.value.filter(key => !key.includes(event.handlerTag.toString()));
    return touchableItem === null || touchableItem === void 0 ? void 0 : (_touchableItem$onEnd = touchableItem.onEnd) === null || _touchableItem$onEnd === void 0 ? void 0 : _touchableItem$onEnd.call(touchableItem, event);
  });
  useEffect(() => {
    return () => {
      touchableRefs.value = {};
    };
  }, [touchableRefs]);
  return /*#__PURE__*/React.createElement(GestureDetector, {
    gesture: mainGesture
  }, /*#__PURE__*/React.createElement(SkiaCanvas, props, /*#__PURE__*/React.createElement(TouchHandlerContext.Provider, {
    value: touchableRefs
  }, children)));
};
export { Canvas };
//# sourceMappingURL=canvas.js.map