import { useCallback, useEffect, useId } from 'react';
import { useTouchHandlerContext } from '../canvas/context';
import { getCirclePath } from '../utils/get-circle-path';
import { getRectPath, getRoundedRectPath } from '../utils/get-rect-path';
import { unwrapAnimatedValue, unwrapAnimatedValueObject } from '../utils/unwrap-animated-value';
export const getSkiaPath = (key, props) => {
  'worklet';

  const unwrappedProps = unwrapAnimatedValueObject(props);
  switch (key) {
    case 'Circle':
      return getCirclePath(unwrappedProps);
    case 'Rect':
      return getRectPath(unwrappedProps);
    case 'RoundedRect':
      return getRoundedRectPath(unwrappedProps);
    case 'Path':
      return unwrappedProps.path;
    default:
      return null;
  }
};
const withTouchableHandler = (Component, componentName) => {
  return _ref => {
    let {
      onStart: onStartProp,
      onActive: onActiveProp,
      onEnd: onEndProp,
      touchablePath,
      ...props
    } = _ref;
    const id = useId();
    const ref = useTouchHandlerContext();
    const onStart = useCallback(event => {
      'worklet';

      return onStartProp === null || onStartProp === void 0 ? void 0 : onStartProp(event);
    }, [onStartProp]);
    const onActive = useCallback(event => {
      'worklet';

      return onActiveProp === null || onActiveProp === void 0 ? void 0 : onActiveProp(event);
    }, [onActiveProp]);
    const onEnd = useCallback(event => {
      'worklet';

      return onEndProp === null || onEndProp === void 0 ? void 0 : onEndProp(event);
    }, [onEndProp]);
    const isPointInPath = useCallback(point => {
      'worklet';

      if (touchablePath) {
        return unwrapAnimatedValue(touchablePath).contains(point.x, point.y);
      }
      if (!componentName) return false;
      const path = getSkiaPath(componentName, props);
      if (!path) {
        throw Error('No touchablePath provided');
      }
      return path.contains(point.x, point.y);
    }, [props, touchablePath]);
    useEffect(() => {
      ref.value = {
        [`id:${id}`]: {
          isPointInPath,
          onStart,
          onActive,
          onEnd
        },
        ...ref.value
      };
      // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [id, isPointInPath, onActive, onEnd, onStart]);
    useEffect(() => {
      return () => {
        var _ref$value;
        (_ref$value = ref.value) === null || _ref$value === void 0 ? true : delete _ref$value[`id:${id}`];
      };
    }, [id, props, ref, touchablePath]);
    return Component(props);
  };
};
export { withTouchableHandler };
//# sourceMappingURL=with-touchable-handler.js.map